#include <SDL/SDL.h>
#include <string.h>
#include <malloc.h>
#include <windows.h>
#include <mmsystem.h>
#include <mmreg.h>

#include "SDL_dibaudioIn.h"

#define NUM_BUFFERS 2
WAVEHDR *wHeader[NUM_BUFFERS]={NULL,NULL};

HWAVEIN __hWaveIn=NULL;

WAVEHDR *CreateWaveHeader(Uint32 buflen)
{
	WAVEHDR *hdr=(WAVEHDR *)malloc(sizeof(WAVEHDR));

	if(hdr==NULL)	return NULL;
	memset(hdr,0,sizeof(WAVEHDR));

	hdr->dwBufferLength=buflen;
	hdr->dwFlags=0;
	hdr->lpData=HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,buflen);
	
	return(hdr);
}

void CALLBACK waveInProc(HWAVEIN hwi,UINT uMsg,DWORD dwInstance,
						 DWORD dwParam1,DWORD dwParam2)
{
	SDL_AudioSpec *spec=(SDL_AudioSpec *)dwInstance;
	void (*CallBack)(void *,Uint8 *, int);
	WAVEHDR *header;


	switch(uMsg)
	{
	case WIM_CLOSE:
		break;

	case WIM_OPEN:
		break;

	case WIM_DATA:
		CallBack=spec->callback;
		header=(WAVEHDR *)dwParam1;

		if(header->dwBytesRecorded)
			CallBack(spec->userdata,header->lpData,header->dwBytesRecorded);

		header->dwBytesRecorded = 0;
	    header->dwFlags &= ~WHDR_DONE;

		waveInAddBuffer(hwi,header,sizeof(WAVEHDR));		

		break;
	}
}

int SDL_OpenAudioIn(SDL_AudioSpec *requested, SDL_AudioSpec *result)
{	
	Uint32 n=0;
	SDL_AudioSpec *nSpec;
	WAVEFORMATEX f_ex;		

	if(requested==NULL)	return -1;

	if(waveInGetNumDevs()==0)
	{
		fprintf(stderr,"No audio devices, open failed.\n");
		return(-1);
	}
	
	memset(&f_ex,0,sizeof(f_ex));		

	switch(requested->format)
	{
	case AUDIO_U16:
	case AUDIO_S16:
		f_ex.wBitsPerSample=16;
		break;

	case AUDIO_U8:
	case AUDIO_S8:
		f_ex.wBitsPerSample=8;
		break;
	}

	f_ex.nChannels=requested->channels;
	f_ex.wFormatTag=WAVE_FORMAT_PCM;
	f_ex.nSamplesPerSec=requested->freq;
	f_ex.nBlockAlign=(f_ex.wBitsPerSample*f_ex.nChannels)/8;
	f_ex.nAvgBytesPerSec=f_ex.nSamplesPerSec*f_ex.nBlockAlign;

	nSpec=(SDL_AudioSpec *)malloc(sizeof(SDL_AudioSpec));
	memcpy(nSpec,requested,sizeof(SDL_AudioSpec));

	if(waveInOpen(&__hWaveIn,WAVE_MAPPER,&f_ex,(DWORD)waveInProc,(DWORD)nSpec,CALLBACK_FUNCTION)!=MMSYSERR_NOERROR)
	{
		fprintf(stderr,"Couldn't open.\n\n");
		free(nSpec);
		return(-1);
	}

	if(result)
	{
		memcpy(result,requested,sizeof(SDL_AudioSpec));

		result->size=requested->samples*f_ex.nBlockAlign;
		result->silence=0;

		switch(requested->format)
		{
		case AUDIO_U16:
		case AUDIO_S16:
			result->format=AUDIO_S16;
			break;

		case AUDIO_U8:
		case AUDIO_S8:
			result->format=AUDIO_U8;
			break;
		}
	}

	for(n=0; n<NUM_BUFFERS; n++)
	{
		wHeader[n]=CreateWaveHeader(requested->samples*f_ex.nBlockAlign);
		waveInPrepareHeader(__hWaveIn,wHeader[n],sizeof(WAVEHDR));
		waveInAddBuffer(__hWaveIn,wHeader[n],sizeof(WAVEHDR));
	}

	return(0);
}

void SDL_PauseAudioIn(int pause)
{
	if(pause)
	{
		waveInStop(__hWaveIn);
	}
	else
	{
		waveInStart(__hWaveIn);
	}
}

void SDL_CloseAudioIn()
{
	int n;
	waveInReset(__hWaveIn);

	for(n=0; n<NUM_BUFFERS; n++)
	{
		waveInUnprepareHeader(__hWaveIn,wHeader[n],sizeof(WAVEHDR));
		HeapFree(GetProcessHeap(),0,wHeader[n]->lpData);
		free(wHeader[n]);
		wHeader[n]=NULL;
	}

	waveInClose(__hWaveIn);
}
